/*
 *      b a l t r e e e  _ c
 *
 *  This contains routines that are specific
 *  to the implementation of balanced binary trees
 *  and are suitable for inclusion into a RLL.
 *
 *
 * (c) Copyright 1996-1997 David J. Walker
 *     Email:  d.j.walker@x400.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  12 May 96   DJW   - First version
 */

#include <liblist.h>
#include "liblistx.h"
#include <stdlib.h>


#ifndef NO
#define NO 0
#endif /* NO */
#ifndef YES
#define YES (!0)
#endif /* YES */
#ifndef MAXIMUM
#define MAXIMUM(x,y)  (((x) < (y)) ? (y) :(x))
#endif /* MAXIMUM */
#ifndef BOOL
#define BOOL int
#endif /* BOOL */


/*===================================================== BALTREE_ADDNODE */
static
void	BALTREE_AddNode (_baltreeh_t treelist, _baltree_t newnode, _baltree_t listnode)
/*		~~~~~~~~~~~~~~~
** This void is the core of HTBTree.c . It will
**       1/ add a new element to the tree at the right place
**          so that the tree remains sorted
**       2/ balance the tree to be as fast as possible when reading it
**----------------------------------------------------------------------*/
{
    _baltree_t father_of_element;
    _baltree_t added_element;
    _baltree_t forefather_of_element;
    _baltree_t father_of_forefather;
    int father_found,top_found;
    int depth,depth2,corrections;

    /*  'father_of_element' is a pointer to the structure that is the father of 
     *                      the new object "object".
     *  'added_element'     is a pointer to the structure that contains or will 
     *                      contain the new object "object".
     *  'father_of_forefather' and 'forefather_of_element'
     *                      are pointers that are used to modify the depths of 
     *                      upper elements, when needed.
     *
     *  'father_found'      indicates by a value 0 when the future father of 
     *                      "object" is found.
     *  'top_found'         indicates by a value 0 when, in case of a 
     *                      difference of depths < 2, the top of the tree is 
     *                      encountered and forbids any further try to balance 
     *                      the tree.
     *  'corrections'       is an integer used to avoid infinite loops in cases
     *                      such as:
     *
     *                       3                        3
     *                    4                              4
     *                     5                            5
     *
     *         3 is used here to show that it need not be the top of the tree.
     */

    /*
    ** 1/ Adding of the element to the binary tree
    */

    if (listnode == NULL)
    {
        /*
         *  List empty
         */
        treelist->nodes = newnode;

        newnode->up = NULL;
        newnode->left = NULL;
        newnode->right = NULL;
        newnode->left_depth = 0;
        newnode->right_depth = 0;
        return;
    }
    else
    {   
        father_found = YES;
        father_of_element = listnode;
        added_element = NULL;
        father_of_forefather = NULL;
        forefather_of_element = NULL;      
        while (father_found)
        {
            if (_LIST_Compare  ( (list_t)treelist, (node_t)newnode, (node_t)father_of_element) < 0)
            {
                if (father_of_element->left != NULL)
                    father_of_element = father_of_element->left;
                else 
                {
                    father_found = NO;
                    father_of_element->left = newnode;

                    added_element = father_of_element->left;
                    added_element->up = father_of_element;
                    added_element->left = NULL;
                    added_element->left_depth = 0;
                    added_element->right = NULL;
                    added_element->right_depth = 0;
                }
            }
            if (_LIST_Compare ((list_t)treelist, (node_t)newnode, (node_t)father_of_element) >= 0)
            {
                if (father_of_element->right != NULL) 
                    father_of_element = father_of_element->right;
                else 
                {  
                    father_found = NO;
                    father_of_element->right = newnode;
                    added_element = father_of_element->right;
                    added_element->up = father_of_element;
                    added_element->left = NULL;
                    added_element->left_depth = 0;
                    added_element->right = NULL;
                    added_element->right_depth = 0;       
                }
            }
        }
            /*
            ** Changing of all depths that need to be changed
            */
        father_of_forefather = father_of_element;
        forefather_of_element = added_element;
        do
        {
            if (father_of_forefather->left == forefather_of_element)
            {
                depth = father_of_forefather->left_depth;
                father_of_forefather->left_depth = 1 
                            + MAXIMUM(forefather_of_element->right_depth,
                                  forefather_of_element->left_depth);
                depth2 = father_of_forefather->left_depth;
            }
            else
            {
                depth = father_of_forefather->right_depth;
                father_of_forefather->right_depth = 1
                            + MAXIMUM(forefather_of_element->right_depth,
                                  forefather_of_element->left_depth);
                depth2 = father_of_forefather->right_depth;
            }
            forefather_of_element = father_of_forefather;
            father_of_forefather = father_of_forefather->up;
        } while ((depth != depth2) && (father_of_forefather != NULL));
        


            /*
            ** 2/ Balancing the binary tree, if necessary
            ** Bugs in this part have been fixed in March 94  -  AS
            */
        top_found = YES;
        corrections = 0;
        while ((top_found) && (corrections < 7))
        {
            if ((abs(father_of_element->left_depth
                      - father_of_element->right_depth)) < 2)
            {
                if (father_of_element->up != NULL) 
                    father_of_element = father_of_element->up;
                else top_found = NO;
            }
            else
            {                /* We start the process of balancing */

                corrections = corrections + 1;
                    /* 
                    ** corrections is an integer used to avoid infinite 
                    ** loops in cases such as:
                    **
                    **             3                        3
                    **          4                              4
                    **           5                            5
                    **
                    ** 3 is used to show that it need not be the top of the tree
                    ** But let's avoid these two exceptions anyhow 
                    ** with the two following conditions (March 94 - AS)
                    */

                if ((father_of_element->left == NULL) 
                    && (father_of_element->right->right == NULL) 
                    && (father_of_element->right->left->left == NULL) 
                    && (father_of_element->right->left->right == NULL)) 
                    corrections = 7;

                if ((father_of_element->right == NULL) 
                    && (father_of_element->left->left == NULL) 
                    && (father_of_element->left->right->right == NULL) 
                    && (father_of_element->left->right->left == NULL))
                    corrections = 7;
 

                if (father_of_element->left_depth > father_of_element->right_depth)
                {
                    added_element = father_of_element->left;
                    father_of_element->left_depth = added_element->right_depth;
                    added_element->right_depth = 1
                                    + MAXIMUM(father_of_element->right_depth,
                                          father_of_element->left_depth);
                    if (father_of_element->up != NULL)
                    {
                        /* Bug fixed in March 94  */
                        BOOL first_time;

                        father_of_forefather = father_of_element->up;
                        forefather_of_element = added_element;
                        first_time = YES;
                        do 
                        {
                            if (father_of_forefather->left
                                 == forefather_of_element->up)
                              {
                                  depth = father_of_forefather->left_depth;
                                  if (first_time)
                                  {
                                      father_of_forefather->left_depth = 1
                                          + MAXIMUM(forefather_of_element->left_depth,
                                                  forefather_of_element->right_depth);
                                        first_time = NO;
                                   }
                                   else
                                       father_of_forefather->left_depth = 1
                                           + MAXIMUM(forefather_of_element->up->left_depth,
                                              forefather_of_element->up->right_depth);

                                depth2 = father_of_forefather->left_depth;
                            }
                            else
                            {
                                depth = father_of_forefather->right_depth;
                                if (first_time)
                                {
                                    father_of_forefather->right_depth = 1
                                      + MAXIMUM(forefather_of_element->left_depth,
                                               forefather_of_element->right_depth);
                                    first_time = NO;
                                }                               
                                else
                                    father_of_forefather->right_depth = 1
                                      + MAXIMUM(forefather_of_element->up->left_depth,
                                           forefather_of_element->up->right_depth);
                                depth2 = father_of_forefather->right_depth;
                            }
                            forefather_of_element = forefather_of_element->up;
                            father_of_forefather = father_of_forefather->up;
                        } while ((depth != depth2) && 
                                 (father_of_forefather != NULL));
                        father_of_forefather = father_of_element->up;
                        if (father_of_forefather->left == father_of_element)
                        {
                            /*
                            **                   3                       3
                            **               4                       5
                            ** When tree   5   6        becomes    7    4
                            **            7 8                          8 6
                            **
                            ** 3 is used to show that it may not be the top of the
                            ** tree.
                            */ 
                            father_of_forefather->left = added_element;
                            father_of_element->left = added_element->right;
                            added_element->right = father_of_element;
                        }
                        if (father_of_forefather->right == father_of_element)
                        {
                            /*
                            **          3                       3
                            **               4                       5
                            ** When tree   5   6        becomes    7    4
                            **            7 8                          8 6
                            **
                            ** 3 is used to show that it may not be the top of the
                            ** tree
                            */
                            father_of_forefather->right = added_element;
                            father_of_element->left = added_element->right;
                            added_element->right = father_of_element;
                        }
                        added_element->up = father_of_forefather;
                    }
                    else
                    {
                        /*
                        **
                        **               1                       2
                        ** When tree   2   3        becomes    4    1
                        **            4 5                          5 3
                        **
                        ** 1 is used to show that it is the top of the tree    
                        */
                        added_element->up = NULL;
                        father_of_element->left = added_element->right;
                        added_element->right = father_of_element;
                    }
                    father_of_element->up = added_element;
                    if (father_of_element->left != NULL)
                        father_of_element->left->up = father_of_element;
                }
                else
                {
                    added_element = father_of_element->right;
                    father_of_element->right_depth = added_element->left_depth;
                    added_element->left_depth = 1 + 
                            MAXIMUM(father_of_element->right_depth,
                                father_of_element->left_depth);
                    if (father_of_element->up != NULL)
                        /* Bug fixed in March 94  */
                    {
                        BOOL first_time;

                        father_of_forefather = father_of_element->up;
                        forefather_of_element = added_element;
                        first_time = YES;
                        do 
                        {
                            if (father_of_forefather->left 
                                == forefather_of_element->up)
                            {
                                depth = father_of_forefather->left_depth;
                                if (first_time)
                                {
                                    father_of_forefather->left_depth = 1
                                       + MAXIMUM(forefather_of_element->left_depth,
                                               forefather_of_element->right_depth);
                                    first_time = NO;
                                }
                                else
                                    father_of_forefather->left_depth = 1
                                      + MAXIMUM(forefather_of_element->up->left_depth,
                                          forefather_of_element->up->right_depth);
                                depth2 = father_of_forefather->left_depth;
                            }
                            else
                            {
                                depth = father_of_forefather->right_depth;
                                if (first_time)
                                {
                                    father_of_forefather->right_depth = 1
                                       + MAXIMUM(forefather_of_element->left_depth,
                                               forefather_of_element->right_depth);
                                    first_time = NO;
                                }
                                else
                                    father_of_forefather->right_depth = 1
                                      + MAXIMUM(forefather_of_element->up->left_depth,
                                           forefather_of_element->up->right_depth);
                                depth2 = father_of_forefather->right_depth;
                            }
                            father_of_forefather = father_of_forefather->up;
                            forefather_of_element = forefather_of_element->up;
                        } while ((depth != depth2) && 
                                 (father_of_forefather != NULL));
                        father_of_forefather = father_of_element->up;
                        if (father_of_forefather->left == father_of_element)
                        {
                            /*
                            **                    3                       3
                            **               4                       6
                            ** When tree   5   6        becomes    4    8
                            **                7 8                 5 7
                            **
                            ** 3 is used to show that it may not be the top of the
                            ** tree.
                            */
                            father_of_forefather->left = added_element;
                            father_of_element->right = added_element->left;
                            added_element->left = father_of_element;
                        }
                        if (father_of_forefather->right == father_of_element)
                        {
                            /*
                            **           3                      3
                            **               4                       6
                            ** When tree   5   6        becomes    4    8
                            **                7 8                 5 7
                            **
                            ** 3 is used to show that it may not be the top of the
                            ** tree
                            */
                            father_of_forefather->right = added_element;
                            father_of_element->right = added_element->left;
                            added_element->left = father_of_element;
                        }
                        added_element->up = father_of_forefather;
                    }
                    else
                    {
                        /*
                        **
                        **               1                       3
                        ** When tree   2   3        becomes    1    5
                        **                4 5                 2 4
                        **
                        ** 1 is used to show that it is the top of the tree.
                        */
                        added_element->up = NULL;
                        father_of_element->right = added_element->left;
                        added_element->left = father_of_element;
                    }
                    father_of_element->up = added_element;
                    if (father_of_element->right != NULL)
                        father_of_element->right->up = father_of_element;
                }
            }
        }
        while (father_of_element->up != NULL)
        {
            father_of_element = father_of_element->up;
        }
        treelist->nodes = father_of_element;
    }
}


/*======================================================== BALTREE_AFTER */
static 
int     BALTREE_After     (list_t list, node_t newnode, node_t oldnode)
/*      ~~~~~~~~~~~~~
 *  Add a node after the given node.
 *-----------------------------------------------------------------------*/
{
	register _baltreeh_t baltreelist = (_baltreeh_t)list;
	register _baltree_t  baltreenewnode = (_baltree_t)newnode;
	register _baltree_t  baltreeoldnode = (_baltree_t)oldnode;

    LIBASSERT ((baltreelist->nodes == NULL && baltreeoldnode == NULL)
          ||(baltreelist->nodes != NULL && baltreeoldnode != NULL) );

    LIBDEBUG("BALTREE_After", \
           "'newnode' appears to already be linked in to a list", \
           baltreenewnode->up == NULL && baltreenewnode->left == NULL && baltreenewnode->right == NULL)

    BALTREE_AddNode (baltreelist, baltreenewnode, baltreeoldnode);

    return (LIST_ERROR_NONE);
}


/*========================================================= BALTREE_ADD */
static 
int     BALTREE_Add       (list_t list, node_t node)
/*      ~~~~~~~~~~~
 *  Add a node at the correct point.
 *----------------------------------------------------------------------*/
{
	register _baltreeh_t  baltreelist = (_baltreeh_t)list;
	register _baltree_t  baltreenode = (_baltree_t)node;

    LIBDEBUG ("BALTREE_Add", \
           "'node' appears to already be linked in to a list", \
           baltreenode->up == NULL && baltreenode->left == NULL && baltreenode->right == NULL)

    BALTREE_AddNode(baltreelist, baltreenode, baltreelist->nodes);

    return (LIST_ERROR_NONE);
}


/*========================================================================
 *
 *  This is the list of vectored functions
 *  that are used to support this list type.
 *
 *------------------------------------------------------------------------*/

static struct _listmaster master =
        {
        &_LIST_ListCount,
        &_LIST_BtreeFirst,
        &_LIST_BtreeLast,
        &_LIST_BtreeNext,
        &_LIST_BtreePrevious,
        &_LIST_CloneList,
        &_LIST_Destroy,
        &_LIST_NewNode,
        &BALTREE_Add,
        &BALTREE_After,
        &_LIST_BtreeRemove,
        &_LIST_BtreeFind,
        LIST_TYPE_BALTREE,
#ifdef DEBUG
        &_LISTbaltree,
        "Balanced Binary Tree"
#endif /* DEBUG */
        };

/*
 *  This is the default settings for a new
 *  object of this type.
 */
struct _listclass _LISTbaltree =
        {
        &master,                    /* Pointer to master class data */
        sizeof(struct _baltreeh),   /* Size of the list header */
        sizeof(struct _baltree),    /* Size of the node header */
        0,                          /* Size of user node data */
        0,                          /* Clone count */
        NULL,                       /* User node Initialise */
        NULL,                       /* User node Destroy */
        NULL,                       /* user node Compare */
        };

