/*
 *      L I S T _ D e s t r o y
 *
 *  This function is called when it is required
 *  to destroy a list.
 *
 *
 * (c) Copyright 1996-1997 David J. Walker
 *     Email:  d.j.walker@x400.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  18 May 96   DJW   - First version
 */

#include <liblist.h>
#include "liblistx.h"


/*======================================================== _LIST_DESTROY */
extern
int  _LIST_Destroy  (list_t * list)
/*  ~~~~~~~~~~~~~~
 *  This is an internal support routine that is used
 *  from various list type specific destruction methods.
 *-----------------------------------------------------------------------*/
{
    node_t  listnode;
    register listclass_t  listclass;
    register _listmaster_t listmaster;

    listclass = (*list)->listclass;
    listmaster = listclass->master;
    while ((listnode = listmaster->first(*list)) != NULL)
    {
        int     result;
        /*
         *  Remove the node from the list
         *  (this cannot fail ?)
         */
        result = listmaster->remove(*list, _LIST_REALNODE(*list,listnode));
        LIBASSERT (result == LIST_ERROR_NONE);
        /*
         *  Kill off the node
         *  Check no error as if so we had better
         *  abandon before catastrophes happen.
         */
        if (LIST_ERROR_NONE != (result = LIST_FreeNode (*list,&listnode)))
        {
            return result;
        }
    }
	_LIST_UnClone (*list);
	*list = NULL;
    return LIST_ERROR_NONE;
}

/*========================================================== LIST_DESTORY */
extern
int  LIST_Destroy  (list_t * plist)
/*
 *  This is the publically available version of the
 *  list destruction routine.
 *------------------------------------------------------------------------*/
{
	if (plist && *plist == NULL)
	{
		return LIST_ERROR_NONE;
	}
#ifdef LISTTRACK
    if (_LIST_TrackDelete (*plist) != LIST_ERROR_NONE)
    {
        return LIST_ERROR_NOTFOUND;
    }
#endif /* LISTTRACK */
    LIBDEBUG ("LIST_Destroy", \
           "list does not point to a valid list instance", \
           _LIST_ListCheck(*plist))

    return (*plist)->listclass->master->destroy(plist);
}

