/*
 *      L I S T _ N e w N o d e
 *
 *  This function is called when it is required
 *  to create a new nodelist.
 *
 *
 * (c) Copyright 1996-1997 David J. Walker
 *     Email:  d.j.walker@x400.icl.co.uk
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  18 May 96   DJW   - First version
 */

#include <liblist.h>
#include "liblistx.h"
#include <stdlib.h>

#ifdef DEBUG
extern
void    _LIST_InitNode  (list_t list, node_t node)
{
    /*
     *  Set the node ID check.
     *  N.B.  This assumes that ALL node types
     *      have the ID field as the first field.
     */
    ((_nodecheck_t)node)->list = list;
    ((_nodecheck_t)node)->type = list->listclass;
    return;
}
#endif /* DEBUG */


/*
 *  Internal support version of the new
 *  node creation routine which has the
 *  variable parameter list dereferenced.
 */
extern
node_t  _LIST_NewNode (list_t list, va_list ap)
{
    register node_t  node;
    register node_t  nodeuser;
    register listclass_t listclass;

    listclass = list->listclass;

    node = calloc (1,listclass->sizenode + listclass->sizeuser);
    if (node == NULL)
    {
		errno = LIST_ERROR_NOMEMORY;
        return NULL;
    }
#ifdef DEBUG
    _LIST_InitNode (list,node);
#endif /* DEBUG */

    nodeuser = _LIST_USERNODE(list,node);
    /*
     *  Check for a user supplied
     *  initialisation routine.
     */
    if (listclass->initnode)
    {
        int     result;

        result = listclass->initnode(nodeuser,ap);
        /*
         *  If user routine returns error
         *  then we must tidy and propgate
         *  the error back to the user.
         */
        if (result != LIST_ERROR_NONE)
        {
            free (node);
            return (NULL);
        }
    }

    return (nodeuser);
}

/*
 *  Public version of new node routine
 *  This simply dereferences the parameter
 *  list and then calls the internal routine
 */
extern 
node_t  LIST_NewNode (list_t list, ...)
{
    node_t  node;
    va_list ap;

    LIBDEBUG ("LIST_NewNode", \
           "'list' parameter is not a valid list instance", \
           _LIST_ListCheck(list))

    va_start(ap,list);
    node = list->listclass->master->nodenew(list, ap);
    va_end(ap);
    return (node);
}

